/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: Launcher.java 100 2006-03-15 09:44:57Z ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.system.launch;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.fractal.adl.Factory;
import org.objectweb.fractal.adl.FactoryFactory;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.jmx.agent.Admin;
import org.objectweb.fractal.rmi.registry.NamingService;
import org.objectweb.fractal.rmi.registry.Registry;
import org.objectweb.fractal.util.Fractal;

import org.objectweb.petals.kernel.Kernel;

/**
 * @author Rafael Marins
 * @version $Rev: 100 $ $Date: 2006-03-15 10:44:57 +0100 (mer., 15 mars 2006) $
 */
public class Launcher {

	private Factory factory;

	private Component kernelComposite;

	private NamingService naming;

	/**
	 * 
	 */
	public Launcher() {
		super();
	}

	private void run(String[] args) {

		// check required fractal properties
		System.setProperty("fractal.provider", "org.objectweb.fractal.julia.Julia");
		System.setProperty("julia.loader", "org.objectweb.fractal.julia.loader.DynamicLoader");
		System.setProperty("julia.config", "julia.cfg");

		// steps to start Petals
		try {
			initializeKernel();
			
			getKernel().boot();
			
			// automatically expose all component server interfaces for JMX Admin at startup 
			String componentName = "agent";
			ContentController contentController = Fractal
			.getContentController(kernelComposite);
			Component agent = getComponentByName(contentController,
					componentName);
			String interfaceName = "admin";
			Admin agentAdmin = (Admin) agent.getFcInterface(interfaceName);
			agentAdmin.expose();
			

		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}

	private void initializeKernel() throws Exception {

		Map<String,Object> context = new HashMap<String,Object>();

//		ClassLoader cl = Loaders.getCommonLoader();
//		context.put("classloader", cl);

		// Get registry first, otherwise create a new one (localhost)
		naming = Registry.getRegistry();

		// Create Fractal ADL factories
		factory = FactoryFactory.getFactory(FactoryFactory.FRACTAL_BACKEND,
				context);

		// Create composite from ADL
		String compositeName = "PetalsJMX";
		kernelComposite = (Component) factory.newComponent(compositeName,
				context);

		// Register the composite in the Petals to use Fractal Explorer
		try {
			naming.bind(compositeName, kernelComposite);
		} catch (Exception e) {
			// TODO fix loading local registry for components registrations

//			Registry.createRegistry(Registry.DEFAULT_PORT);
//			naming = Registry.getRegistry();
//
//			naming.bind(compositeName, kernelComposite);
		}

		// Start the composite
		LifeCycleController lcc = Fractal
				.getLifeCycleController(kernelComposite);
		lcc.startFc();

	}

	private Kernel getKernel() throws Exception {

		ContentController contentController = Fractal
				.getContentController(kernelComposite);

		/*
		 * Every Petals-Fractal ADL configuration should provide a runnable
		 * deployer component that it bound to the initiators components.
		 */
		String componentName = "kernel-manager";
		String interfaceName = "kernel";

		Component comp = getComponentByName(contentController,
				componentName);

		Kernel k = (Kernel) comp.getFcInterface(interfaceName);

		return k;
	}

	/**
	 * A utility function allowing to get a component from a content controller
	 * 
	 * @param cc
	 *            ContentController
	 * @param name
	 *            component name
	 * @return
	 */
	private static Component getComponentByName(
			ContentController contentController, String name) {

		// The component to be returned
		Component c = null;

		// List content controller subcomponents
		Component[] comps = contentController.getFcSubComponents();

		for (int i = 0; i < comps.length; i++) {

			Component component = comps[i];

			try {
				String componentName = Fractal.getNameController(component)
						.getFcName();

				if (componentName.equals(name)) {
					c = component;
					break;
				}
			} catch (NoSuchInterfaceException e) {
				// Do nothing
				e.printStackTrace();
			}
		}

		return c;
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {

		try {
            Launcher launcher = new Launcher();
            launcher.run(args);
        } catch (Exception e) {
            e.printStackTrace();
        }
	}

}
